// Copyright (c) 2025 shibedrill
// SPDX-License-Identifier: GPL-3.0-or-later

#![allow(clippy::missing_safety_doc)]
#![allow(dead_code)]

use core::arch::asm;

pub unsafe fn halt() {
    unsafe {
        asm!("hlt");
    }
}

pub unsafe fn port_read_u8(port: u16) -> u8 {
    let result: u8;
    unsafe {
        asm! {
            "in al dx",
            in("dx") port,
            out("al") result
        }
    }
    result
}

pub unsafe fn port_read_u16(port: u16) -> u16 {
    let result: u16;
    unsafe {
        asm! {
            "in ax dx",
            in("dx") port,
            out("ax") result
        }
    }
    result
}

pub unsafe fn port_read_u32(port: u16) -> u32 {
    let result: u32;
    unsafe {
        asm! {
            "in eax dx",
            in("dx") port,
            out("eax") result
        }
    }
    result
}

pub unsafe fn port_write_u8(port: u16, data: u8) {
    unsafe {
        asm! {
            "out dx, al",
            in("dx") port,
            in("al") data,
        }
    }
}

pub unsafe fn port_write_u16(port: u16, data: u16) {
    unsafe {
        asm! {
            "out dx, ax",
            in("dx") port,
            in("ax") data,
        }
    }
}

pub unsafe fn port_write_u32(port: u16, data: u32) {
    unsafe {
        asm! {
            "out dx eax",
            in("dx") port,
            in("eax") data,
        }
    }
}
